@@ -94,7 +94,7 @@ class PdfObject(PdfObjectProtocol):
         raise Exception("clone PdfObject")
 
     def _reference_clone(
-        self, clone: Any, pdf_dest: PdfWriterProtocol
+        self, clone: Any, pdf_dest: PdfWriterProtocol, force_duplicate: bool = False
     ) -> PdfObjectProtocol:
         """
         Reference the object within the _objects of pdf_dest only if
@@ -110,25 +110,31 @@ class PdfObject(PdfObjectProtocol):
           The clone
         """
         try:
-            if clone.indirect_reference.pdf == pdf_dest:
+            if not force_duplicate and clone.indirect_reference.pdf == pdf_dest:
                 return clone
         except Exception:
             pass
-        if hasattr(self, "indirect_reference"):
+        # if hasattr(clone, "indirect_reference"):
+        try:
             ind = self.indirect_reference
-            i = len(pdf_dest._objects) + 1
-            if ind is not None:
-                if id(ind.pdf) not in pdf_dest._id_translated:
-                    pdf_dest._id_translated[id(ind.pdf)] = {}
-                if ind.idnum in pdf_dest._id_translated[id(ind.pdf)]:
-                    obj = pdf_dest.get_object(
-                        pdf_dest._id_translated[id(ind.pdf)][ind.idnum]
-                    )
-                    assert obj is not None
-                    return obj
-                pdf_dest._id_translated[id(ind.pdf)][ind.idnum] = i
-            pdf_dest._objects.append(clone)
-            clone.indirect_reference = IndirectObject(i, 0, pdf_dest)
+        except AttributeError:
+            return clone
+        i = len(pdf_dest._objects) + 1
+        if ind is not None:
+            if id(ind.pdf) not in pdf_dest._id_translated:
+                pdf_dest._id_translated[id(ind.pdf)] = {}
+            if (
+                not force_duplicate
+                and ind.idnum in pdf_dest._id_translated[id(ind.pdf)]
+            ):
+                obj = pdf_dest.get_object(
+                    pdf_dest._id_translated[id(ind.pdf)][ind.idnum]
+                )
+                assert obj is not None
+                return obj
+            pdf_dest._id_translated[id(ind.pdf)][ind.idnum] = i
+        pdf_dest._objects.append(clone)
+        clone.indirect_reference = IndirectObject(i, 0, pdf_dest)
         return clone
 
     def get_object(self) -> Optional["PdfObject"]:
@@ -153,7 +159,9 @@ class NullObject(PdfObject):
         ignore_fields: Union[Tuple[str, ...], List[str], None] = (),
     ) -> "NullObject":
         """Clone object into pdf_dest."""
-        return cast("NullObject", self._reference_clone(NullObject(), pdf_dest))
+        return cast(
+            "NullObject", self._reference_clone(NullObject(), pdf_dest, force_duplicate)
+        )
 
     def write_to_stream(
         self, stream: StreamType, encryption_key: Union[None, str, bytes]
@@ -194,7 +202,8 @@ class BooleanObject(PdfObject):
     ) -> "BooleanObject":
         """Clone object into pdf_dest."""
         return cast(
-            "BooleanObject", self._reference_clone(BooleanObject(self.value), pdf_dest)
+            "BooleanObject",
+            self._reference_clone(BooleanObject(self.value), pdf_dest, force_duplicate),
         )
 
     def __eq__(self, __o: object) -> bool:
@@ -366,7 +375,10 @@ class FloatObject(float, PdfObject):
         ignore_fields: Union[Tuple[str, ...], List[str], None] = (),
     ) -> "FloatObject":
         """Clone object into pdf_dest."""
-        return cast("FloatObject", self._reference_clone(FloatObject(self), pdf_dest))
+        return cast(
+            "FloatObject",
+            self._reference_clone(FloatObject(self), pdf_dest, force_duplicate),
+        )
 
     def myrepr(self) -> str:
         if self == 0:
@@ -410,7 +422,10 @@ class NumberObject(int, PdfObject):
         ignore_fields: Union[Tuple[str, ...], List[str], None] = (),
     ) -> "NumberObject":
         """Clone object into pdf_dest."""
-        return cast("NumberObject", self._reference_clone(NumberObject(self), pdf_dest))
+        return cast(
+            "NumberObject",
+            self._reference_clone(NumberObject(self), pdf_dest, force_duplicate),
+        )
 
     def as_numeric(self) -> int:
         return int(repr(self).encode("utf8"))
@@ -459,7 +474,9 @@ class ByteStringObject(bytes, PdfObject):
         """Clone object into pdf_dest."""
         return cast(
             "ByteStringObject",
-            self._reference_clone(ByteStringObject(bytes(self)), pdf_dest),
+            self._reference_clone(
+                ByteStringObject(bytes(self)), pdf_dest, force_duplicate
+            ),
         )
 
     @property
@@ -505,7 +522,9 @@ class TextStringObject(str, PdfObject):
         obj = TextStringObject(self)
         obj.autodetect_pdfdocencoding = self.autodetect_pdfdocencoding
         obj.autodetect_utf16 = self.autodetect_utf16
-        return cast("TextStringObject", self._reference_clone(obj, pdf_dest))
+        return cast(
+            "TextStringObject", self._reference_clone(obj, pdf_dest, force_duplicate)
+        )
 
     autodetect_pdfdocencoding = False
     autodetect_utf16 = False
@@ -587,7 +606,10 @@ class NameObject(str, PdfObject):
         ignore_fields: Union[Tuple[str, ...], List[str], None] = (),
     ) -> "NameObject":
         """Clone object into pdf_dest."""
-        return cast("NameObject", self._reference_clone(NameObject(self), pdf_dest))
+        return cast(
+            "NameObject",
+            self._reference_clone(NameObject(self), pdf_dest, force_duplicate),
+        )
 
     def write_to_stream(
         self, stream: StreamType, encryption_key: Union[None, str, bytes]
@@ -623,7 +645,7 @@ class NameObject(str, PdfObject):
                 sin = sin[:i] + unhexlify(sin[i + 1 : i + 3]) + sin[i + 3 :]
                 i = sin.find(b"#", i + 1)
             except ValueError:
-                # if the 2 characters after # can not be converted to hexa
+                # if the 2 characters after # can not be converted to hex
                 # we change nothing and carry on
                 i = i + 1
         return sin
